package fr.ill.ics.cameo;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.security.CodeSource;
import java.util.ArrayList;

public class TestSelector {

	public static Process startServer(String config) {

		System.out.println("Starting server");
		
		String jarFileName = "";
		
		CodeSource codeSource = TestSelector.class.getProtectionDomain().getCodeSource();
		try {
			File jarFile = new File(codeSource.getLocation().toURI().getPath());
			jarFileName = jarFile.toString();
			
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
			System.exit(1);
		}
		
		ArrayList<String> commandList = new ArrayList<String>();
		commandList.add("java");
		
		String javaLibraryPath = System.getProperty("java.library.path");
		if (javaLibraryPath != null) {
			commandList.add("-Djava.library.path=" + javaLibraryPath);
		}
		commandList.add("-classpath");
		commandList.add(jarFileName);
		commandList.add("fr.ill.ics.cameo.server.Server");
		commandList.add(config);

		// Prepare the command
		String command[] = new String[commandList.size()];
		commandList.toArray(command);
		
		ProcessBuilder builder = new ProcessBuilder(command);
		builder.redirectErrorStream(true);
		
		try {
			System.out.println("Server started");
			return builder.start();
			
		} catch (IOException e) {
			e.printStackTrace();
		}

		return null;
	}
	
	public static void startApplication(Server server, String appName, String[] appArgs) {
		
		Application.Instance instance = server.start(appName, appArgs, Option.OUTPUTSTREAM);

		if (instance.exists()) {

			// start output thread
			OutputStreamSocket streamSocket = instance.getOutputStreamSocket();

			OutputPrintThread outputThread = null;

			// the socket can be null if the application is already
			// terminated

			if (streamSocket == null) {
				System.out.println("The application " + instance.getNameId() + " has no output stream.");
			} else {
				outputThread = new OutputPrintThread(streamSocket);
				outputThread.start();
			}

			System.out.println("Starting application");
			int result = instance.waitFor();
			System.out.println("Finished application");

			// Terminate the thread and the server.
			if (streamSocket != null) {
				outputThread.waitFor();
			}
		} else {
			System.out.println("No application");
		}
	}
	
	public static void main(String[] args) {
		
		if (args.length == 0) {
			System.out.println("Usage: <application name> <number of iterations>");
			System.exit(1);
		}
		
		int argsIndex = 0;
		
		// Define the server process.
		Process serverProcess = null;
		
		// Start the server.
		serverProcess = startServer(args[0]);
				
		Server server = new Server("tcp://localhost:10000"); // change depending on config
	
		if (server.isAvailable()) {
			System.out.println("Server is available");
		}
		
		argsIndex = 1;
		
		try {
			boolean all = false;
			String appName;
			String[] appArgs;
			
			ArrayList<String> apps = new ArrayList<String>();
			
			if (args.length < argsIndex + 1) {
				
				System.out.println("Available applications:");
				
				for (String app : apps) {
					System.out.println("\t" + app);
				}	
								
				return;
			}
			else {
				appName = args[argsIndex];
				
				if (appName.equals("all")) {
					apps.add("testsimplejava");
					apps.add("testveryfastjava");
					apps.add("teststopjava");
					apps.add("startresjava");
					apps.add("starterrorjava");
					apps.add("startstoragejava");
					apps.add("startpubjava");
					apps.add("startnpubjava");
					apps.add("startrepjava");
					apps.add("startreqjava");
					apps.add("canceljava");
					
					apps.add("testsimplecpp");
					apps.add("testveryfastcpp");
					apps.add("startstopcpp");
					apps.add("startrescpp");
					apps.add("starterrorcpp");
					apps.add("startstoragecpp");
					apps.add("startpubcpp");
					apps.add("startnpubcpp");
					apps.add("startrepcpp");
					apps.add("cancelcpp");
					
					apps.add("testsimplepy");
				}
				else if (appName.equals("java")) {
					apps.add("testsimplejava");
					apps.add("testveryfastjava");
					apps.add("teststopjava");
					apps.add("startresjava");
					apps.add("starterrorjava");
					apps.add("startstoragejava");
					apps.add("startpubjava");
					apps.add("startnpubjava");
					apps.add("startrepjava");
					apps.add("startreqjava");
					apps.add("canceljava");
				}
				else if (appName.equals("cpp")) {
					apps.add("testsimplecpp");
					apps.add("testveryfastcpp");
					apps.add("teststopcpp");
					apps.add("startrescpp");
					apps.add("starterrorcpp");
					apps.add("startstoragecpp");
					apps.add("startpubcpp");
					apps.add("startnpubcpp");
					apps.add("startrepcpp");
					apps.add("cancelcpp");
				}
				else if (appName.equals("python")) {
					apps.add("testsimplepy");
					apps.add("testveryfastpy");
					apps.add("teststoppy");
				}
				else {
					apps.add(appName);
				}
				
				appArgs = new String[args.length - 1 - argsIndex];
				for (int i = 0; i < args.length - 1 - argsIndex; ++i) {
					appArgs[i] = args[argsIndex + i + 1];
				}
			}

			// Test all applications or only one.
			for (String app : apps) {
				startApplication(server, app, appArgs);	
			}
			
		} finally {
			// Terminate the server.
			server.terminate();

			// If the server process exists.
			if (serverProcess != null) {
				// Destroy the server process.
				serverProcess.destroyForcibly();
				
				// Waiting for the server that should not return without
				// interruption.
				try {
					serverProcess.waitFor();
					
				} catch (InterruptedException e) {
					// Do nothing.
				}
			}
		}
	}
}
